Package: adverSCarial
Title: adverSCarial, generate and analyze the vulnerability of
        scRNA-seq classifier to adversarial attacks
Version: 1.11.0
Authors@R: c(
    person("Ghislain", "FIEVET", , "ghislain.fievet@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0337-7327")),
    person("Sébastien", "HERGALANT", role = "aut",
    email="sebastien.hergalant@inserm.fr", comment=c(ORCID = "0000-0001-8456-7992")))
Description: 
    adverSCarial is an R Package designed for generating and analyzing the vulnerability of scRNA-seq
    classifiers to adversarial attacks. The package is versatile and provides a format for integrating
    any type of classifier. It offers functions for studying and generating two types of attacks,
    single gene attack and max change attack. The single-gene attack involves making a small modification
    to the input to alter the classification. The max-change attack involves making a large modification
    to the input without changing its classification. The CGD attack is based on an estimated gradient descent.
    against adversarial attacks.
    The package provides a comprehensive solution for evaluating the robustness of scRNA-seq classifiers
    against adversarial attacks.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
biocViews: Software, SingleCell, Transcriptomics, Classification
Suggests: knitr, RUnit, BiocGenerics, TENxPBMCData, CHETAH, stringr,
        LoomExperiment
Imports: gtools, S4Vectors, methods, DelayedArray
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/adverSCarial
git_branch: devel
git_last_commit: 0818fc6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:10:19 UTC; biocbuild
Author: Ghislain FIEVET [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0337-7327>),
  Sébastien HERGALANT [aut] (ORCID:
    <https://orcid.org/0000-0001-8456-7992>)
Maintainer: Ghislain FIEVET <ghislain.fievet@gmail.com>
Depends: R (>= 3.5.0)
