# Generated by roxygen2: do not edit by hand

export(MClassifier)
export(advCGD)
export(advChar)
export(advGridMinChange)
export(advList)
export(advMaxChange)
export(advModifications)
export(advRandWalkMinChange)
export(advSingleGene)
export(advTraining)
export(getSignGenes)
export(matrixFromSCE)
export(maxChangeOverview)
export(predictWithNewValue)
export(sceConvertToHGNC)
export(singleGeneOverview)
exportClasses(advChar)
exportClasses(advList)
importFrom(methods,setMethod)
importFrom(methods,setClass)